package javax.swing;

import javax.swing.plaf.ColorUIResource;
import java.lang.reflect.Constructor;
import sun.swing.SwingUtilities2;
import java.security.PrivilegedAction;
import java.security.AccessController;
import java.security.AccessControlContext;
import java.beans.PropertyChangeListener;
import sun.reflect.misc.MethodUtil;
import java.lang.reflect.Method;
import sun.reflect.misc.ReflectUtil;
import javax.swing.plaf.ComponentUI;
import java.awt.Dimension;
import java.awt.Insets;
import javax.swing.border.Border;
import java.awt.Color;
import java.awt.Font;
import java.util.Enumeration;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import sun.util.CoreResourceBundleControl;
import java.util.HashMap;
import java.util.Map;
import java.util.Locale;
import java.util.Vector;
import javax.swing.event.SwingPropertyChangeSupport;
import java.util.Hashtable;

public class UIDefaults extends Hashtable<Object, Object>
{
    private static final Object PENDING;
    private SwingPropertyChangeSupport changeSupport;
    private Vector<String> resourceBundles;
    private Locale defaultLocale;
    private Map<Locale, Map<String, Object>> resourceCache;
    
    public UIDefaults() {
        this(700, 0.75f);
    }
    
    public UIDefaults(int initialCapacity, float loadFactor) {
        super(initialCapacity, loadFactor);
        this.defaultLocale = Locale.getDefault();
        this.resourceCache = new HashMap();
    }
    
    public UIDefaults(Object[] array) {
        super(array.length / 2);
        this.defaultLocale = Locale.getDefault();
        for (int i = 0; i < array.length; i += 2) {
            super.put(array[i], array[i + 1]);
        }
    }
    
    @Override
    public Object get(Object o) {
        Object fromHashtable = this.getFromHashtable(o);
        return (fromHashtable != null) ? fromHashtable : this.getFromResourceBundle(o, null);
    }
    
    private Object getFromHashtable(Object o) {
        Object value = super.get(o);
        if (value != UIDefaults.PENDING && !(value instanceof UIDefaults.ActiveValue) && !(value instanceof UIDefaults.LazyValue)) {
            return value;
        }
        synchronized (this) {
            value = super.get(o);
            if (value == UIDefaults.PENDING) {
                while (true) {
                    try {
                        this.wait();
                    }
                    catch (InterruptedException ex) {}
                    value = super.get(o);
                    if (value != UIDefaults.PENDING) {
                        break;
                    }
                    continue;
                }
                return value;
            }
            if ((value instanceof UIDefaults.LazyValue)) {
                super.put(o, UIDefaults.PENDING);
            }
            else if (!(value instanceof UIDefaults.ActiveValue)) {
                return value;
            }
        }
        if ((value instanceof UIDefaults.LazyValue)) {
            try {
                value = ((UIDefaults.LazyValue)value).createValue(this);
            }
            finally {
                synchronized (this) {
                    if (value == null) {
                        super.remove(o);
                    }
                    else {
                        super.put(o, value);
                    }
                    this.notifyAll();
                }
            }
            return value;
        }
        value = ((UIDefaults.ActiveValue)value).createValue(this);
        return value;
    }
    
    public Object get(Object o, Locale locale) {
        Object fromHashtable = this.getFromHashtable(o);
        return (fromHashtable != null) ? fromHashtable : this.getFromResourceBundle(o, locale);
    }
    
    private Object getFromResourceBundle(Object o, Locale defaultLocale) {
        if (this.resourceBundles == null || this.resourceBundles.isEmpty() || !(o instanceof String)) {
            return null;
        }
        if (defaultLocale == null) {
            if (this.defaultLocale == null) {
                return null;
            }
            defaultLocale = this.defaultLocale;
        }
        synchronized (this) {
            return this.getResourceCache(defaultLocale).get(o);
        }
    }
    
    private Map<String, Object> getResourceCache(Locale locale) {
        Map<String, Object> map = (Map)this.resourceCache.get(locale);
        if (map == null) {
            map = new UIDefaults.TextAndMnemonicHashMap();
            for (int i = this.resourceBundles.size() - 1; i >= 0; --i) {
                String s = (String)this.resourceBundles.get(i);
                try {
                    CoreResourceBundleControl rbControlInstance = CoreResourceBundleControl.getRBControlInstance(s);
                    ResourceBundle resourceBundle;
                    if (rbControlInstance != null) {
                        resourceBundle = ResourceBundle.getBundle(s, locale, rbControlInstance);
                    }
                    else {
                        resourceBundle = ResourceBundle.getBundle(s, locale, ClassLoader.getSystemClassLoader());
                    }
                    Enumeration<String> keys = resourceBundle.getKeys();
                    while (keys.hasMoreElements()) {
                        String key = (String)keys.nextElement();
                        if (map.get(key) == null) {
                            map.put(key, resourceBundle.getObject(key));
                        }
                        else {
                            continue;
                        }
                    }
                }
                catch (MissingResourceException ex) {}
            }
            this.resourceCache.put(locale, map);
            return map;
        }
        return map;
    }
    
    @Override
    public Object put(Object o, Object value) {
        Object o2 = (value == null) ? super.remove(o) : super.put(o, value);
        if ((o instanceof String)) {
            this.firePropertyChange((String)o, o2, value);
            return o2;
        }
        return o2;
    }
    
    public void putDefaults(Object[] array) {
        for (int i = 0; i < array.length; i += 2) {
            Object value = array[i + 1];
            if (value == null) {
                super.remove(array[i]);
            }
            else {
                super.put(array[i], value);
            }
        }
        this.firePropertyChange("UIDefaults", null, null);
    }
    
    public Font getFont(Object o) {
        Object value = this.get(o);
        return (value instanceof Font) ? ((Font)value) : null;
    }
    
    public Font getFont(Object o, Locale locale) {
        Object value = this.get(o, locale);
        return (value instanceof Font) ? ((Font)value) : null;
    }
    
    public Color getColor(Object o) {
        Object value = this.get(o);
        return (value instanceof Color) ? ((Color)value) : null;
    }
    
    public Color getColor(Object o, Locale locale) {
        Object value = this.get(o, locale);
        return (value instanceof Color) ? ((Color)value) : null;
    }
    
    public Icon getIcon(Object o) {
        Object value = this.get(o);
        return (value instanceof Icon) ? ((Icon)value) : null;
    }
    
    public Icon getIcon(Object o, Locale locale) {
        Object value = this.get(o, locale);
        return (value instanceof Icon) ? ((Icon)value) : null;
    }
    
    public Border getBorder(Object o) {
        Object value = this.get(o);
        return (value instanceof Border) ? ((Border)value) : null;
    }
    
    public Border getBorder(Object o, Locale locale) {
        Object value = this.get(o, locale);
        return (value instanceof Border) ? ((Border)value) : null;
    }
    
    public String getString(Object o) {
        Object value = this.get(o);
        return (value instanceof String) ? ((String)value) : null;
    }
    
    public String getString(Object o, Locale locale) {
        Object value = this.get(o, locale);
        return (value instanceof String) ? ((String)value) : null;
    }
    
    public int getInt(Object o) {
        Object value = this.get(o);
        return (value instanceof Integer) ? ((int)value) : 0;
    }
    
    public int getInt(Object o, Locale locale) {
        Object value = this.get(o, locale);
        return (value instanceof Integer) ? ((int)value) : 0;
    }
    
    public boolean getBoolean(Object o) {
        Object value = this.get(o);
        return value instanceof Boolean && (boolean)value;
    }
    
    public boolean getBoolean(Object o, Locale locale) {
        Object value = this.get(o, locale);
        return value instanceof Boolean && (boolean)value;
    }
    
    public Insets getInsets(Object o) {
        Object value = this.get(o);
        return (value instanceof Insets) ? ((Insets)value) : null;
    }
    
    public Insets getInsets(Object o, Locale locale) {
        Object value = this.get(o, locale);
        return (value instanceof Insets) ? ((Insets)value) : null;
    }
    
    public Dimension getDimension(Object o) {
        Object value = this.get(o);
        return (value instanceof Dimension) ? ((Dimension)value) : null;
    }
    
    public Dimension getDimension(Object o, Locale locale) {
        Object value = this.get(o, locale);
        return (value instanceof Dimension) ? ((Dimension)value) : null;
    }
    
    public Class<? extends ComponentUI> getUIClass(String s, ClassLoader classLoader) {
        try {
            String s2 = (String)this.get(s);
            if (s2 != null) {
                ReflectUtil.checkPackageAccess(s2);
                Class<Object> clazz = (Class)this.get(s2);
                if (clazz == null) {
                    if (classLoader == null) {
                        clazz = SwingUtilities.loadSystemClass(s2);
                    }
                    else {
                        clazz = classLoader.loadClass(s2);
                    }
                    if (clazz != null) {
                        this.put(s2, clazz);
                    }
                }
                return clazz;
            }
        }
        catch (ClassNotFoundException ex) {
            return null;
        }
        catch (ClassCastException ex2) {
            return null;
        }
        return null;
    }
    
    public Class<? extends ComponentUI> getUIClass(String s) {
        return this.getUIClass(s, null);
    }
    
    protected void getUIError(String str) {
        System.err.println("UIDefaults.getUI() failed: " + str);
        try {
            throw new Error();
        }
        catch (Throwable t) {
            t.printStackTrace();
            return;
        }
    }
    
    public ComponentUI getUI(JComponent component) {
        Object value = this.get("ClassLoader");
        Class<Object> uiClass = this.getUIClass(component.getUIClassID(), (value != null) ? ((ClassLoader)value) : component.getClass().getClassLoader());
        Object invoke = null;
        if (uiClass == null) {
            this.getUIError("no ComponentUI class for: " + component);
        }
        else {
            try {
                Method method = (Method)this.get(uiClass);
                if (method == null) {
                    method = uiClass.getMethod("createUI", new Class[] { JComponent.class });
                    this.put(uiClass, method);
                }
                invoke = MethodUtil.invoke(method, null, new Object[] { component });
            }
            catch (NoSuchMethodException ex) {
                this.getUIError("static createUI() method not found in " + uiClass);
            }
            catch (Exception obj) {
                this.getUIError("createUI() failed for " + component + " " + obj);
            }
        }
        return (ComponentUI)invoke;
    }
    
    public synchronized void addPropertyChangeListener(PropertyChangeListener listener) {
        if (this.changeSupport == null) {
            this.changeSupport = new SwingPropertyChangeSupport(this);
        }
        this.changeSupport.addPropertyChangeListener(listener);
    }
    
    public synchronized void removePropertyChangeListener(PropertyChangeListener listener) {
        if (this.changeSupport != null) {
            this.changeSupport.removePropertyChangeListener(listener);
        }
    }
    
    public synchronized PropertyChangeListener[] getPropertyChangeListeners() {
        if (this.changeSupport == null) {
            return new PropertyChangeListener[0];
        }
        return this.changeSupport.getPropertyChangeListeners();
    }
    
    protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
        if (this.changeSupport != null) {
            this.changeSupport.firePropertyChange(propertyName, oldValue, newValue);
        }
    }
    
    public synchronized void addResourceBundle(String s) {
        if (s == null) {
            return;
        }
        if (this.resourceBundles == null) {
            this.resourceBundles = new Vector(5);
        }
        if (!this.resourceBundles.contains(s)) {
            this.resourceBundles.add(s);
            this.resourceCache.clear();
        }
        return;
    }
    
    public synchronized void removeResourceBundle(String o) {
        if (this.resourceBundles != null) {
            this.resourceBundles.remove(o);
        }
        this.resourceCache.clear();
    }
    
    public void setDefaultLocale(Locale defaultLocale) {
        this.defaultLocale = defaultLocale;
    }
    
    public Locale getDefaultLocale() {
        return this.defaultLocale;
    }
    
    static {
        PENDING = new Object();
    }
    
    public static class ProxyLazyValue implements UIDefaults.LazyValue
    {
        private AccessControlContext acc;
        private String className;
        private String methodName;
        private Object[] args;
        
        public ProxyLazyValue(String s) {
            this(s, (String)null);
        }
        
        public ProxyLazyValue(String s, String s2) {
            this(s, s2, (Object[])null);
        }
        
        public ProxyLazyValue(String s, Object[] array) {
            this(s, null, array);
        }
        
        public ProxyLazyValue(String className, String methodName, Object[] array) {
            this.acc = AccessController.getContext();
            this.className = className;
            this.methodName = methodName;
            if (array != null) {
                this.args = (Object[])array.clone();
            }
        }
        
        @Override
        public Object createValue(UIDefaults uiDefaults) {
            if (this.acc == null && System.getSecurityManager() != null) {
                throw new SecurityException("null AccessControlContext");
            }
            return AccessController.doPrivileged(new PrivilegedAction<Object>() {
                @Override
                public Object run() {
                    try {
                        Object o;
                        if (uiDefaults == null || !((o = uiDefaults.get("ClassLoader")) instanceof ClassLoader)) {
                            o = Thread.currentThread().getContextClassLoader();
                            if (o == null) {
                                o = ClassLoader.getSystemClassLoader();
                            }
                        }
                        ReflectUtil.checkPackageAccess(UIDefaults.ProxyLazyValue.this.className);
                        Class<Object> forName = Class.forName(UIDefaults.ProxyLazyValue.this.className, true, (ClassLoader)o);
                        SwingUtilities2.checkAccess(forName.getModifiers());
                        if (UIDefaults.ProxyLazyValue.this.methodName != null) {
                            return MethodUtil.invoke(forName.getMethod(UIDefaults.ProxyLazyValue.this.methodName, UIDefaults.ProxyLazyValue.access$400(UIDefaults.ProxyLazyValue.this, UIDefaults.ProxyLazyValue.this.args)), forName, UIDefaults.ProxyLazyValue.this.args);
                        }
                        Constructor<Object> constructor = forName.getConstructor(UIDefaults.ProxyLazyValue.access$400(UIDefaults.ProxyLazyValue.this, UIDefaults.ProxyLazyValue.this.args));
                        SwingUtilities2.checkAccess(constructor.getModifiers());
                        return constructor.newInstance(UIDefaults.ProxyLazyValue.this.args);
                    }
                    catch (Exception ex) {
                        return null;
                    }
                }
            }, this.acc);
        }
        
        private Class[] getClassArray(Object[] array) {
            Class[] array2 = null;
            if (array != null) {
                array2 = new Class[array.length];
                for (int i = 0; i < array.length; ++i) {
                    if ((array[i] instanceof Integer)) {
                        array2[i] = Integer.TYPE;
                    }
                    else if ((array[i] instanceof Boolean)) {
                        array2[i] = Boolean.TYPE;
                    }
                    else if ((array[i] instanceof ColorUIResource)) {
                        array2[i] = Color.class;
                    }
                    else {
                        array2[i] = array[i].getClass();
                    }
                }
                return array2;
            }
            return array2;
        }
        
        private String printArgs(Object[] array) {
            String concat = "{";
            if (array != null) {
                for (int i = 0; i < array.length - 1; ++i) {
                    concat = concat.concat(array[i] + ",");
                }
                return concat.concat(array[array.length - 1] + "}");
            }
            return concat.concat("}");
        }
        
        static Class[] access$400(UIDefaults.ProxyLazyValue proxyLazyValue, Object[] array) {
            return proxyLazyValue.getClassArray(array);
        }
    }
    
    public static class LazyInputMap implements UIDefaults.LazyValue
    {
        private Object[] bindings;
        
        public LazyInputMap(Object[] bindings) {
            this.bindings = bindings;
        }
        
        @Override
        public Object createValue(UIDefaults uiDefaults) {
            if (this.bindings != null) {
                return LookAndFeel.makeInputMap(this.bindings);
            }
            return null;
        }
    }
    
    private static class TextAndMnemonicHashMap extends HashMap<String, Object>
    {
        static final String AND_MNEMONIC = "AndMnemonic";
        static final String TITLE_SUFFIX = ".titleAndMnemonic";
        static final String TEXT_SUFFIX = ".textAndMnemonic";
        
        @Override
        public Object get(Object key) {
            Object value = super.get(key);
            if (value == null) {
                boolean n = false;
                String string = key.toString();
                String s = null;
                if (string.endsWith("AndMnemonic")) {
                    return null;
                }
                if (string.endsWith(".mnemonic")) {
                    s = this.composeKey(string, 9, ".textAndMnemonic");
                }
                else if (string.endsWith("NameMnemonic")) {
                    s = this.composeKey(string, 12, ".textAndMnemonic");
                }
                else if (string.endsWith("Mnemonic")) {
                    s = this.composeKey(string, 8, ".textAndMnemonic");
                    n = true;
                }
                if (s != null) {
                    Object o2 = super.get(s);
                    if (o2 == null && n) {
                        o2 = super.get(this.composeKey(string, 8, ".titleAndMnemonic"));
                    }
                    return (o2 == null) ? null : this.getMnemonicFromProperty(o2.toString());
                }
                else {
                    if (string.endsWith("NameText")) {
                        s = this.composeKey(string, 8, ".textAndMnemonic");
                    }
                    else if (string.endsWith(".nameText")) {
                        s = this.composeKey(string, 9, ".textAndMnemonic");
                    }
                    else if (string.endsWith("Text")) {
                        s = this.composeKey(string, 4, ".textAndMnemonic");
                    }
                    else if (string.endsWith("Title")) {
                        s = this.composeKey(string, 5, ".titleAndMnemonic");
                    }
                    if (s != null) {
                        Object value2 = super.get(s);
                        return (value2 == null) ? null : this.getTextFromProperty(value2.toString());
                    }
                    if (string.endsWith("DisplayedMnemonicIndex")) {
                        Object o = super.get(this.composeKey(string, 22, ".textAndMnemonic"));
                        if (o == null) {
                            o = super.get(this.composeKey(string, 22, ".titleAndMnemonic"));
                        }
                        return (o == null) ? null : this.getIndexFromProperty(o.toString());
                    }
                }
            }
            return value;
        }
        
        String composeKey(String s, int n, String str) {
            return s.substring(0, s.length() - n) + str;
        }
        
        String getTextFromProperty(String s) {
            return s.replace("&", "");
        }
        
        String getMnemonicFromProperty(String s) {
            int index = s.indexOf(38);
            if (0 <= index && index < s.length() - 1) {
                return Integer.toString((int)Character.toUpperCase(s.charAt(index + 1)));
            }
            return null;
        }
        
        String getIndexFromProperty(String s) {
            int index = s.indexOf(38);
            return (index == -1) ? null : Integer.toString(index);
        }
    }
    
    public interface ActiveValue
    {
        public abstract Object createValue(UIDefaults p0);
    }
    
    public interface LazyValue
    {
        public abstract Object createValue(UIDefaults p0);
    }
}
